﻿using UnityEngine;
using System.Collections;
using UnityEngine.Networking;

public partial class CombatEffects : NetworkBehaviour 
{
	public GameObject weaponSlot;
	public GameObject damagePrefab;
	public Transform selectionDecal;
	public GameObject damageExplosion;
	public GameObject deathExplosion;
	public AudioSource shootSound;
	public AudioSource hitSound;
	public ParticleSystem levelUp;
	public Combat combat;
	
	Transform muzzle;
	ParticleEmitter damageEmitter;
	float colliderRadiusHeuristic;
	NetworkInstanceId targetId;
		
	public override void OnStartClient () 
	{
		if (weaponSlot != null) {
			muzzle = weaponSlot.transform.Find("muzzleFlash");
			if (muzzle != null) {
				muzzle.gameObject.SetActive(false);
				muzzle.GetComponent<Light>().enabled = false;
			}
		}
		
		if (damagePrefab) {
			
			GameObject effect = (GameObject)Instantiate(damagePrefab, Vector3.zero, Quaternion.identity);
			effect.transform.parent = transform;
			effect.transform.localPosition = Vector3.zero;
			damageEmitter = effect.GetComponent<ParticleEmitter>();
			
			Vector2 tempSize = new Vector2(GetComponent<Collider>().bounds.extents.x,GetComponent<Collider>().bounds.extents.z);
			colliderRadiusHeuristic = tempSize.magnitude * 0.1f;
		}
		
		if (selectionDecal) {
			selectionDecal.GetComponent<Renderer>().enabled = false;
		}
		Firing(false);
		
		combat.EventFire += Firing;
		combat.EventAttack += Attack;
		combat.EventTakeDamage += TakeDamage;
		combat.EventSetLevel += GainLevel;
		combat.EventDie += Die;
		combat.EventSetTarget += SetTarget;
	}
	
	public void Attack()
	{
		muzzle.gameObject.SetActive(true);
		muzzle.GetComponent<Light>().enabled = true;
		shootSound.loop = true;
		shootSound.Play();
		Invoke ("StopAttack", 0.2f);
	}
	
	public void StopAttack()
	{
		muzzle.gameObject.SetActive(false);
		muzzle.GetComponent<Light>().enabled = false;
		shootSound.Stop();	
	}
	
	public void Firing(bool flag)
	{
		//Debug.Log ("FIRE ON CLIENT " + flag);
	/*
		if (flag)
		{
			muzzle.gameObject.SetActive(true);
			muzzle.light.enabled = true;
			shootSound.loop = true;
			shootSound.Play();
		}
		else
		{
			muzzle.gameObject.SetActive(false);
			muzzle.light.enabled = false;
			shootSound.Stop();
		}
	*/
	}
	
	public void TakeDamage(int amount, Vector3 fromDirection, DamageType dt)
	{
		// Show damage effect if there is one
		if (damageEmitter) {
			damageEmitter.transform.rotation = Quaternion.LookRotation (fromDirection, Vector3.up);
			Vector3 dir = fromDirection.normalized;
			dir.y = 0.0f;
			damageEmitter.transform.position = transform.position + colliderRadiusHeuristic * dir;
			damageEmitter.Emit();
		}
		if (hitSound != null) {
			hitSound.Play();
		}
		if (damageExplosion != null)
		{
			GameObject x = (GameObject)Instantiate(damageExplosion, transform.position + fromDirection.normalized*3, Quaternion.identity);
			GameObject.Destroy(x, 1.0f);
		}
	}
	
	public void GainLevel(uint level)
	{
		if (levelUp != null) {
			levelUp.Play();
		}
	}
	
	public void SetTarget(NetworkInstanceId id)
	{
		if (!isLocalPlayer) {
			return;
		}
		if (!targetId.IsEmpty() && id != targetId) {
			// untarget current target
			GameObject target = ClientScene.FindLocalObject(targetId);
			if (target != null) {
				CombatEffects fx = target.GetComponent<CombatEffects>();
				if (fx != null) {
					fx.selectionDecal.GetComponent<Renderer>().enabled = false;
				}
			}
		}
		
		if (!id.IsEmpty()) {
			//target new target
			GameObject target = ClientScene.FindLocalObject(id);
			if (target != null) {
				CombatEffects fx = target.GetComponent<CombatEffects>();
				if (fx != null) {
					fx.selectionDecal.GetComponent<Renderer>().enabled = true;
				}
			}			
		}
		
		targetId = id;
	}
	
	void Die()
	{
		Instantiate(deathExplosion, transform.position, Quaternion.identity);
	}
}
