﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.Networking;

public struct Floatie
{
	public CombatStat stat;
	public 	int amount;
	public Vector3 pos;
	public Color color;
	public float expireTimer;
}

public partial class CombatLabel : NetworkBehaviour {
	
	public Combat combat;
	public Transform label;
	public bool hasMana = false;
	const float labelRange = 100.0f;
	
	List<Floatie> floaties = new List<Floatie>();
	Combat target;
	
	// for drawing UI (ugh)
	public Texture box;
	
	public override void OnStartClient () {
		
		combat.EventSetTarget += SetTarget;
		combat.EventGainXP += GainXP;
		combat.EventTakeDamage += TakeDamage;		
	}
	
	void DrawLabels(Combat combat, Vector3 pos)
	{
		// draw the name with a shadow
		GUI.color = Color.black;
		GUI.Label(new Rect(pos.x-20, Screen.height - pos.y - 30, 100, 40), combat.combatName);
		
		GUI.color = combat.GetTeamColor();
		GUI.Label(new Rect(pos.x-21, Screen.height - pos.y - 29, 100, 40), combat.combatName);		
		
		if (!combat.invulnerable)
		{
			// draw health bar background
			GUI.color = Color.grey;
			GUI.DrawTexture (new Rect(pos.x-26, Screen.height - pos.y -10, 52, 5), box);
			
			// draw health bar amount
			GUI.color = Color.green;
			int healthWidth = (int)((combat.health / (float)combat.maxHealth) * 50);
			GUI.DrawTexture (new Rect(pos.x-25, Screen.height - pos.y -9, healthWidth, 3), box);
		}
		
		if (hasMana)
		{
			// draw mana bar background
			GUI.color = Color.grey;
			GUI.DrawTexture (new Rect(pos.x-26, Screen.height - pos.y -4, 52, 5), box);
			
			// draw mana bar amount
			GUI.color = Color.yellow;
			int manaWidth = (int)((combat.mana / (float)combat.maxMana) * 50);
			GUI.DrawTexture (new Rect(pos.x-25, Screen.height - pos.y -3, manaWidth, 3), box);	
		}
	}
	
	void DrawCooldowns()
	{
	/*
		GUI.color = Color.grey;
		GUI.DrawTexture (new Rect(0,0,160,160), box);
		
		string[] names = {"damage1", "damage2", "regen", "str"};
		// draw cooldowns		
		for (int i=0; i < 4; i++) {
			float diff = 0.0f;
			if (combat.abilityTimer[i] >= Time.time) {
				diff = Time.time - combat.abilityTimer[i];
			}
			string txt = "F" + (i+1) + "(" + names[i] + "): " + diff.ToString();
			//Debug.Log (txt);
			GUI.color = Color.white;
			GUI.Label(new Rect(5, 10 + i*20, 200, 30), txt);		
		}	*/
	}
	
	void DrawBufs(Combat combat, int x, int y)
	{
	/*
		GUI.color = Color.white;
		int by = y;
		foreach(var buf in combat.bufs)
		{
			foreach (var mod in buf.mods) {
				string txt = "stat: " + mod.stat + " amount: " + mod.amount + " expire: " + (Time.time - buf.expireTimer);
				GUI.Label(new Rect(x, by, 400, 30), txt);	
				by += 20;
			}
		}
		*/
	}
	
	void DrawSelection()
	{
		if (target == null) {
			return;
		}
		
		GUI.Label(new Rect(5, 100, 200, 20), "Selected:");
		
		Vector3 bufPos = new Vector3(100, Screen.height -120, 0);
		DrawLabels(target, bufPos);
		
		DrawBufs(target, 5, 140);
		
	}
	
	void DrawFloaties(Vector3 pos)
	{
		if (ClientScene.localPlayers.Count == 0) {
			return;
		}
		
		if (ClientScene.localPlayers[0].gameObject == null)
		{
			Debug.LogError("Local player seems to be invalid (gameObject = null)!");
			return;
		}
		
		Combat player = ClientScene.localPlayers[0].gameObject.GetComponent<Combat>();
		if (player == null) {
			return;
		}
		
		foreach(Floatie f in floaties)
		{
			if (combat == player) {
				GUI.color = Color.red;
			}
			else if (combat.team == player.team)
			{
				GUI.color = Color.yellow;
			}
			else
			{
				GUI.color = Color.green;
			}
			string txt;
			if (f.amount > 0) {
				txt = "+" + f.amount;
			} else {
				txt = "" + f.amount;
			}
			
			if (f.stat == CombatStat.Health) {
				txt = txt + " HP";
			}
			else if (f.stat == CombatStat.Experience) {
				txt = txt + " XP";
				GUI.color = Color.cyan;
			}
			else if (f.stat == CombatStat.Gold) {
				txt = txt + " $$";
				GUI.color = Color.yellow;
			}
			
			GUI.Label(new Rect(pos.x + f.pos.x, Screen.height - (pos.y + f.pos.y), 100, 40), txt);
		}
	}
	
	void OnGUI()
	{

		if (!NetworkClient.active) {
			return;
		}
		
		if (Camera.main != null) {
			
			Transform l = label;
			if (l == null) {
				l = transform;
			}
			
			Vector3 pos = Camera.main.WorldToScreenPoint(l.position);
			if (pos.z > 0 && pos.z < labelRange) {
				DrawLabels(combat, pos);
			}
			
			//if (Mathf.Abs(pos.z) < labelRange)
			//{
				DrawFloaties(pos);
			//}
		}
		
		if (isLocalPlayer)
		{
			DrawCooldowns();
			DrawSelection();
			DrawBufs(combat, 5, 100);			
		}
	}	
	
	public void GainXP(uint amount)
	{
		if (isLocalPlayer) {
			Floatie f = new Floatie();
			f.stat = CombatStat.Experience;
			f.amount = (int)amount;
			f.color = Color.green;
			f.pos = new Vector3(-10,30);
			f.expireTimer = Time.time + 2;
			floaties.Add(f);
		}
	}
	
	public void TakeDamage(int amount, Vector3 fromDirection, DamageType dt)
	{
		Floatie f = new Floatie();
		f.stat = CombatStat.Health;
		f.amount = -amount;
		f.color = Color.green;
		f.pos = new Vector3(-10,30);
		f.expireTimer = Time.time + 2;
		floaties.Add(f);
	}
	
	
	void FixedUpdate()
	{
		// check for expired floaties
		for (int i = floaties.Count - 1; i >= 0; i--)
		{ 
			if (Time.time > floaties[i].expireTimer) {
				floaties.RemoveAt(i);
			}
			else
			{
				Floatie f= floaties[i];
				f.pos.y += 1;
				
				floaties[i] = f;
			}
		}
	}
	
	public void SetTarget(NetworkInstanceId id)
	{
		if (id.IsEmpty())
		{
			target = null;
		}
		else
		{
			GameObject obj = ClientScene.FindLocalObject(id);
			if (obj != null) {
				target = obj.GetComponent<Combat>();
			}
		}
	}
}
