﻿using UnityEngine;
using System.Collections;
using UnityEngine.Networking;

public partial class RTSMove : NetworkBehaviour {

	//float turningSmoothing = 0.8f;
	Plane movementPlane = new Plane(Vector3.up, Vector3.zero);
	Vector3 cursorScreenPosition;
	public Vector3 cursorWorldPosition;
	public Vector3 moveTarget = Vector3.zero;
	
	float maxTurnSpeed = 8.0f;
	float moveForce = 60.0f;
	float topSpeed = 0.0f;
	
	public ParticleSystem groundCursorPrefab;
	ParticleSystem groundCursor;
	
	bool firingOnClient = false;
		
	public Combat combat;
	
	void Start ()
	{
		groundCursor = (ParticleSystem)Instantiate(groundCursorPrefab, Vector3.zero, Quaternion.identity);
		groundCursor.Stop();
	}
	
	Combat GetCursorTarget()
	{
		RaycastHit hit;
		Ray ray = Camera.main.ScreenPointToRay (Input.mousePosition);
		
		if (Physics.Raycast (ray, out hit, 100.0f)){
			//Debug.Log (hit.collider);
			Combat combat = hit.collider.GetComponent<Combat>();
			return combat;
		}
		return null;
	}
	
	void Update()
	{
		if (!isLocalPlayer) {
			return;
		}
		
		if (Input.GetMouseButtonDown(1))
		{
			// right click on unit selects and moves to it, and attacks unit
			Combat combatTarget = GetCursorTarget();
			if (combatTarget != null) {	
				CmdRTSTarget(combatTarget.GetComponent<NetworkIdentity>().netId);
				moveTarget = combatTarget.transform.position;
				
				if (!firingOnClient) {
					CmdRTSStartFiring();
					firingOnClient = true;
				}
				return;
			}				
			
			// right click on ground moves to ground, cancels attack
			cursorScreenPosition = Input.mousePosition;
			cursorWorldPosition = ScreenPointToWorldPointOnPlane (cursorScreenPosition, movementPlane, Camera.main);
			groundCursor.transform.position = cursorWorldPosition;			
			cursorWorldPosition.y = transform.position.y;
			moveTarget = Vector3.zero;

			groundCursor.GetComponent<ParticleSystem>().Play();
			
			if (firingOnClient) {
				CmdRTSStopFiring();
				firingOnClient = false;
			}
		}
		
		if (Input.GetMouseButtonDown(0))
		{
			// left on click on unit selects unit, cancels attack
			Combat combatTarget = GetCursorTarget();
			if (combatTarget != null) {
				CmdRTSTarget(combatTarget.GetComponent<NetworkIdentity>().netId);
				moveTarget = Vector3.zero;

				if (firingOnClient) {
					CmdRTSStopFiring();
					firingOnClient = false;
				}
				return;
			}
			
			// left click on ground de-selected current unit
			if (combat.target != null)
			{
				CmdRTSTarget(NetworkInstanceId.Invalid);
			}
		}
		
		
		if (Input.GetKeyDown(KeyCode.F1)) {
			CmdUseAbility(0);
		}
		if (Input.GetKeyDown(KeyCode.F2)) {
			CmdUseAbility(1);
		}
		if (Input.GetKeyDown(KeyCode.F3)) {
			CmdUseAbility(2);
		}
		if (Input.GetKeyDown(KeyCode.F4)) {
			CmdUseAbility(3);
		}				
	}
	
	// Update is called once per frame
	void FixedUpdate () {
	
		if (NetworkClient.active && !isLocalPlayer) {
			return;
		}
		
		// move to moveTarget object first, or fallback to cursor position
		Vector3 targetPosition = cursorWorldPosition;
		if (moveTarget != Vector3.zero) {
			targetPosition = moveTarget;
		}
		
		// initial state
		if (targetPosition == Vector3.zero) {
			return;
		}
		
		// already at destination
		if ((targetPosition - transform.position).sqrMagnitude < 0.1f) {
			return;
		}
		
		// ensure rotation is flat around y axis
		targetPosition.y = transform.position.y; 
		
		// rotate to face target point
		Quaternion targetDirection = Quaternion.identity;
		targetDirection.SetLookRotation(targetPosition - transform.position, transform.up);
		
		float angle = Quaternion.Angle( transform.rotation, targetDirection );
		if (angle > maxTurnSpeed) {
			float ratio = maxTurnSpeed / angle;
			targetDirection = Quaternion.Slerp(transform.rotation, targetDirection, ratio);
		}
		transform.rotation = targetDirection;
		
		// move towards target point if facing it
		if (Mathf.Abs(angle) < 5) 
		{
			// stop if reached position, or attack range of target
			float stopRange = 4.0f;
			if (moveTarget != Vector3.zero) {
				stopRange = Mathf.Max(stopRange, combat.attackRange);
			}
			if ((targetPosition - transform.position).magnitude  < stopRange) 
			{
				//reached target position
				cursorWorldPosition = transform.position;
				moveTarget = Vector3.zero;
				GetComponent<Rigidbody>().velocity = Vector3.zero;
				GetComponent<Rigidbody>().angularVelocity = Vector3.zero;
				return;
			}
			
			// this is a magic number
			topSpeed = combat.moveSpeed/20;
			
			if (GetComponent<Rigidbody>().velocity.magnitude >= topSpeed) 
			{
				GetComponent<Rigidbody>().velocity = GetComponent<Rigidbody>().velocity.normalized * topSpeed;
			}
			else
			{
				Vector3 force = transform.forward * moveForce;
				GetComponent<Rigidbody>().AddForce(force);		
			}
		}
	}
	
	[Command]
	public void CmdRTSTarget(NetworkInstanceId targetId)
	{
		GameObject target = NetworkServer.FindLocalObject(targetId);
		if (target != null)
		{
			Combat targetCombat = target.GetComponent<Combat>();
			if (targetCombat != null)
			{
				combat.SetTarget(targetCombat);
			}
		}
		else
		{
			combat.SetTarget(null);
		}
	}
	
	[Command]
	public void CmdRTSStartFiring()
	{
		combat.StartFiring();
	}
	
	[Command]
	public void CmdRTSStopFiring()
	{
		combat.StopFiring();
	}
	
	
	[Command]
	public void CmdUseAbility(int id)
	{
		combat.UseAbility(id);
	}
	
	public static Vector3 ScreenPointToWorldPointOnPlane (Vector3 screenPoint, Plane plane, Camera camera)
	{
		// Set up a ray corresponding to the screen position
		Ray ray = camera.ScreenPointToRay (screenPoint);
		
		// Find out where the ray intersects with the plane
		float dist;
		plane.Raycast (ray, out dist);
		return ray.GetPoint (dist);		
	}
	
	
}
