﻿using UnityEngine;
using System.Collections;
using UnityEngine.Networking;


public partial class AITargeting : NetworkBehaviour {
	
	public Combat combat;
	
	float targetTimer;
	float targetInterval = 1.0f;
	const float outOfRange = 1.4f; // ratio of attack range to forget a target
	
	public override void OnStartClient()
	{
		combat.EventSetTarget += SetTarget;
	}
	
	void Update () 
	{
		if (!NetworkServer.active) {
			return;
		}
		if (combat.target == null) {
			if (Time.time > targetTimer) {
				LookForTarget();
				targetTimer = Time.time + targetInterval;
			}
		}
		else
		{
			if (Time.time > targetTimer) {
				if (!ValidateTarget(combat.target)) {
					combat.SetTarget(null);
				}
				targetTimer = Time.time + targetInterval;
			}
		}
	}
	
	
	void LookForTarget()
	{
		Combat[] targets = Combat.GetTargetsInRange(transform.position, combat.attackRange * outOfRange, Team.Any, combat.team);
		if (targets.Length > 0) 
		{
			Combat newTarget = targets[0];
			if (ValidateTarget (newTarget))
			{
				//TODO: closest?
				combat.SetTarget(newTarget);
			}
		}
	}
	
	bool ValidateTarget(Combat targetCombat)
	{
		// object exists?
		// object is alive?
		if (!targetCombat.isAlive)
			return false;
			
		// object in range?
		if (!combat.IsTargetInRange(targetCombat.transform, combat.attackRange * outOfRange)) {
			return false;
		}
		return true;
	}
		
	public void SetTarget(NetworkInstanceId id)
	{
		if (id.IsEmpty())
		{
			combat.target = null;
		}
		else
		{
			GameObject obj = ClientScene.FindLocalObject(id);
			if (obj != null) {
				combat.target = obj.GetComponent<Combat>();
			}
			else
			{
				Debug.LogWarning("Target id " + id + " not found");
			}
		}
	}	
}
