﻿using UnityEngine;
using System.Collections;
using UnityEngine.Networking;

public partial class Creep : NetworkBehaviour {

	public Transform[]  waypoints;
	public GameObject missilePrefab;
		
	public int waypointIndex = 0;
	
	public Vector3 m_MoveTarget;
	
	float walkingSpeed = 12.0f;
	float walkingSnappyness = 50f;
	float turningSmoothing = 0.8f;
	
	Missile activeMissile;	
	public Combat combat;
	
	// Use this for initialization
	public override void OnStartServer () {
		name = "Creep[ " + GetComponent<NetworkIdentity>().netId + " ] " + combat.team;
		combat.EventDie += Die;
		//Debug.Log ("StartServer Creep: " + NetworkIdentity.netId);
	}
	
	void Update () 
	{
		if (isServer && 
		waypoints.Length >= waypointIndex+1 && 
		waypoints[waypointIndex] != null && 
		(waypoints[waypointIndex].position - transform.position).magnitude < 4.0f)
		{
			//Debug.Log (" Creep: " + NetworkIdentity.netId + " hit waypoint " + waypointIndex);
			waypointIndex += 1;
			if (waypoints[waypointIndex] != null) {
				NextWaypoint();
			}
		}
	}
	
	void Die()
	{
		NetworkServer.Destroy(gameObject);
	}
	
	public void NextWaypoint()
	{
		if (waypoints == null || waypoints.Length == 0) {
			return;
		}
		
		Vector3 diff = (waypoints[waypointIndex].position - transform.position).normalized;
		diff.y = 0;
		
		SetMoveDir(diff);
		RpcSetMoveDir(diff);
	}
		
	[ClientRpc]
	public void RpcSetMoveDir(Vector3 dir)
	{
		SetMoveDir (dir);
	}
	
	public void SetMoveDir(Vector3 dir)
	{
		m_MoveTarget = dir;
		GetComponentInChildren<SpiderAnim>().moveDirection = dir;
	}
	
	void FixedUpdate () 
	{
		if (combat.target != null) {
			GetComponent<Rigidbody>().velocity = Vector3.zero;
			
			// TODO: move to within attack range of target?
			if (isServer)
			{
				if (combat.CanAttack ())
				{
					//combat.target.TakeDamage((int)combat.attackDamage, transform.position, DamageType.Physical);
					GameObject obj = (GameObject)Instantiate(missilePrefab, transform.position, transform.rotation); 
					activeMissile = obj.GetComponent<Missile>();
					activeMissile.target = combat.target;
					activeMissile.damage = (int)combat.attackDamage;
					activeMissile.targetId = combat.target.GetComponent<NetworkIdentity>().netId;
					
					NetworkServer.Spawn(obj);
					
					combat.HasAttacked();
				}
			}
		}
		else
		{
			// Handle the movement of the character
			Vector3 targetVelocity  = m_MoveTarget * walkingSpeed;
			Vector3 deltaVelocity  = targetVelocity - GetComponent<Rigidbody>().velocity;
			if (GetComponent<Rigidbody>().useGravity)
				deltaVelocity.y = 0;
			
			GetComponent<Rigidbody>().AddForce (deltaVelocity * walkingSnappyness, ForceMode.Acceleration);
			
			// Setup player to face facingDirection, or if that is zero, then the movementDirection
			Vector3 faceDir = m_MoveTarget;
			
			// Make the character rotate towards the target rotation
			if (faceDir == Vector3.zero) {
				GetComponent<Rigidbody>().angularVelocity = Vector3.zero;
			}
			else {
				float rotationAngle = AngleAroundAxis (transform.forward, faceDir, Vector3.up);
				GetComponent<Rigidbody>().angularVelocity = (Vector3.up * rotationAngle * turningSmoothing);
			}
		}
	}
	
	static float HorizontalAngle (Vector3 direction) {
		return Mathf.Atan2 (direction.x, direction.z) * Mathf.Rad2Deg;
	}
	
	// The angle between dirA and dirB around axis
	static float AngleAroundAxis (Vector3 dirA , Vector3 dirB, Vector3 axis) {
		// Project A and B onto the plane orthogonal target axis
		dirA = dirA - Vector3.Project (dirA, axis);
		dirB = dirB - Vector3.Project (dirB, axis);
		
		// Find (positive) angle between A and B
		float angle = Vector3.Angle (dirA, dirB);
		
		// Return angle multiplied with 1 or -1
		return angle * (Vector3.Dot (axis, Vector3.Cross (dirA, dirB)) < 0 ? -1 : 1);
	}
	

}
