﻿using UnityEngine;
using System.Collections;
using UnityEngine.Networking;

public partial class RandomMove : NetworkBehaviour {

	Vector3 m_MoveDirection;
	
	float walkingSpeed = 2.0f;
	float walkingSnappyness = 50f;
	float turningSmoothing = 0.8f;
		
	public Combat combat;
	
	// Use this for initialization
	void Start () {
		// die event on server only
		if (NetworkServer.active) {
			//combat.eDie += Die;
		}
	}
	
	void Update () 
	{
		if (NetworkServer.active)
		{
			if (Random.Range(0,1500) == 1) 
			{
				Vector3 newDir = new Vector3(Random.Range (-100,100)*0.01f, 0, Random.Range (-100,100)*0.01f);
				RpcSetMoveDir(newDir.normalized);
				SetMoveDir(newDir.normalized);
			}
		}	
	}
	
	void Die()
	{
		NetworkServer.Destroy(gameObject);
	}
	
	[ClientRpc]
	public void RpcSetMoveDir(Vector3 dir)
	{
		SetMoveDir(dir);
	}
	
	public void SetMoveDir(Vector3 dir)
	{
		m_MoveDirection = dir;
		GetComponentInChildren<SpiderAnim>().moveDirection = dir;
	}
	
	void FixedUpdate () 
	{
		// Handle the movement of the character
		Vector3 targetVelocity  = m_MoveDirection * walkingSpeed;
		Vector3 deltaVelocity  = targetVelocity - GetComponent<Rigidbody>().velocity;
		if (GetComponent<Rigidbody>().useGravity)
			deltaVelocity.y = 0;
			
		GetComponent<Rigidbody>().AddForce (deltaVelocity * walkingSnappyness, ForceMode.Acceleration);
		
		// Setup player to face facingDirection, or if that is zero, then the movementDirection
		Vector3 faceDir = m_MoveDirection;
		
		// Make the character rotate towards the target rotation
		if (faceDir == Vector3.zero) {
			GetComponent<Rigidbody>().angularVelocity = Vector3.zero;
		}
		else {
			float rotationAngle = AngleAroundAxis (transform.forward, faceDir, Vector3.up);
			GetComponent<Rigidbody>().angularVelocity = (Vector3.up * rotationAngle * turningSmoothing);
		}
	}
		
	static float HorizontalAngle (Vector3 direction) {
		return Mathf.Atan2 (direction.x, direction.z) * Mathf.Rad2Deg;
	}
		
	// The angle between dirA and dirB around axis
	static float AngleAroundAxis (Vector3 dirA , Vector3 dirB, Vector3 axis) {
		// Project A and B onto the plane orthogonal target axis
		dirA = dirA - Vector3.Project (dirA, axis);
		dirB = dirB - Vector3.Project (dirB, axis);
		
		// Find (positive) angle between A and B
		float angle = Vector3.Angle (dirA, dirB);
		
		// Return angle multiplied with 1 or -1
		return angle * (Vector3.Dot (axis, Vector3.Cross (dirA, dirB)) < 0 ? -1 : 1);
	}
}
